@@ -81,6 +81,7 @@ _initialize_handle_info(genders_t handle)
 {
   handle->magic = GENDERS_MAGIC_NUM;
   handle->is_loaded = 0;
+  handle->flags = GENDERS_FLAG_DEFAULT;
   handle->numnodes = 0;
   handle->numattrs = 0;
   handle->maxattrs = 0;
@@ -284,6 +285,43 @@ genders_perror(genders_t handle, const char *msg)
     fprintf(stderr, "%s: %s\n", msg, errormsg);
 }
 
+int
+genders_get_flags(genders_t handle, unsigned int *flags)
+{
+  if (_genders_handle_error_check(handle) < 0)
+    return -1;
+
+  if (!flags)
+    {
+      handle->errnum = GENDERS_ERR_PARAMETERS;
+      return -1;
+    }
+
+  *flags = handle->flags;
+  handle->errnum = GENDERS_ERR_SUCCESS;
+  return 0;
+}
+
+int
+genders_set_flags(genders_t handle, unsigned int flags)
+{
+  unsigned int mask = (GENDERS_FLAG_DEFAULT
+		       | GENDERS_FLAG_RAW_VALUES);
+
+  if (_genders_handle_error_check(handle) < 0)
+    return -1;
+
+  if (flags & ~mask)
+    {
+      handle->errnum = GENDERS_ERR_PARAMETERS;
+      return -1;
+    }
+
+  handle->flags = flags;
+  handle->errnum = GENDERS_ERR_SUCCESS;
+  return 0;
+}
+
 int 
 genders_getnumnodes(genders_t handle) 
 {
@@ -1545,6 +1583,7 @@ genders_copy(genders_t handle)
     }
 
   handlecopy->is_loaded = handle->is_loaded;
+  handlecopy->flags = handle->flags;
   handlecopy->numnodes = handle->numnodes;
   handlecopy->numattrs = handle->numattrs;
   handlecopy->maxattrs = handle->maxattrs;
@@ -1585,7 +1624,6 @@ genders_copy(genders_t handle)
   if (_genders_copy_fill_attr_index(handle, handlecopy) < 0)
     goto cleanup;
   
-  /* XXX fill each node with attrvalslist and attr_index */
   if (_genders_copy_fill_node_data(handle, handlecopy) < 0)
     goto cleanup;
 
