@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: genders_parsing.c,v 1.37 2010/02/02 00:04:34 chu11 Exp $
+ *  $Id: genders_parsing.c,v 1.37 2010-02-02 00:04:34 chu11 Exp $
  *****************************************************************************
  *  Copyright (C) 2007-2010 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2001-2007 The Regents of the University of California.
@@ -127,6 +127,7 @@ _insert_node(genders_t handle,
   if (n) 
     {
       __list_destroy(n->attrlist);
+      __hash_destroy(n->attrlist_index);
       free(n);
     }
   return NULL;
@@ -135,12 +136,12 @@ _insert_node(genders_t handle,
 /* 
  * _insert_attrval
  *
- * Insert an attrval into the attrvals list.
+ * Insert an attrval into the attrvallist.
  *
  * Returns 0 on success, -1 on error
  */
 static int 
-_insert_attrval(genders_t handle, List attrvals, char *attr, char *val) 
+_insert_attrval(genders_t handle, genders_attrvals_container_t avc, char *attr, char *val) 
 {
   genders_attrval_t av = NULL;
 
@@ -157,7 +158,7 @@ _insert_attrval(genders_t handle, List attrvals, char *attr, char *val)
   else 
     av->val = NULL; 
 
-  __list_append(attrvals, av);
+  __list_append(avc->attrvals, av);
 
   return 0;
  cleanup:
@@ -179,6 +180,7 @@ _insert_attrval(genders_t handle, List attrvals, char *attr, char *val)
  */
 static int 
 _insert_attr(genders_t handle,
+	     List attrslist,
              hash_t *attr_index,
              int *attr_index_size,
              char *attr) 
@@ -191,7 +193,7 @@ _insert_attr(genders_t handle,
 
   /* insert into attrlist */
   __xstrdup(attr_new, attr);
-  __list_append(handle->attrslist, attr_new);
+  __list_append(attrslist, attr_new);
 
   /* insert into attr_index */
 
@@ -214,7 +216,7 @@ _insert_attr(genders_t handle,
 /* 
  * _attr_node_processsing
  *
- * Determine if an attr in the attrvals list already exists for the
+ * Determine if an attr in the attrvallist already exists for the
  * node.  If not insert into the attr_index and node attrlist_index.
  *
  * If line_num > 0, returns 1 if a duplicate exists, 0 if not, -1 on error
@@ -224,20 +226,48 @@ _insert_attr(genders_t handle,
 static int
 _attr_node_processsing(genders_t handle, 
                        genders_node_t n, 
-                       List attrvals,
+		       genders_attrvals_container_t avc,
                        hash_t *attr_index,
                        int line_num,
                        FILE *stream)
 {
   ListIterator attrvals_itr = NULL;
+  List tmpattrlist = NULL;
   genders_attrval_t av = NULL;
   int rv = -1;
 
-  __list_iterator_create(attrvals_itr, attrvals);
+  __list_create(tmpattrlist, NULL);
+
+  /* First check for parse errors */
+  __list_iterator_create(attrvals_itr, avc->attrvals);
   while ((av = list_next(attrvals_itr))) 
     {
-      List l = NULL;
+      /* do not use _genders_find_attrval().  If the attrval already
+       * exists within the node, we don't need the actual entry.  We
+       * just want to know if it's there or not.  We avoid some list
+       * iteration by not using it.
+       */
+      /* Check attribute already listed for this node and on same line */
+      if (hash_find(n->attrlist_index, av->attr)
+	  || list_find_first(tmpattrlist, _genders_list_is_str, av->attr))
+	{
+	  if (line_num > 0) 
+	    {
+	      fprintf(stream, "Line %d: duplicate attribute \"%s\" listed for node \"%s\"\n",
+		      line_num, av->attr, n->name);
+	      rv = 1;
+	    }
+	  handle->errnum = GENDERS_ERR_PARSE;
+	  goto cleanup;
+	}
+      
+      __list_append(tmpattrlist, av->attr);
+    }
 
+  /* If no parse errors, insert everything as needed */
+  list_iterator_reset(attrvals_itr);
+  while ((av = list_next(attrvals_itr))) 
+    {
       /* do not use _genders_find_attrval().  If the attrval already
        * exists within the node, we don't need the actual entry.  We
        * just want to know if it's there or not.  We avoid some list
@@ -254,6 +284,12 @@ _attr_node_processsing(genders_t handle,
 	  handle->errnum = GENDERS_ERR_PARSE;
 	  goto cleanup;
 	}
+    }
+
+  list_iterator_reset(attrvals_itr);
+  while ((av = list_next(attrvals_itr)))
+    {
+      List l = NULL;
 
       /* add node to attr_index */
       /* attribute should already be in the hash */
@@ -273,12 +309,13 @@ _attr_node_processsing(genders_t handle,
       
       __hash_insert(n->attrlist_index,
                     av->attr,
-                    attrvals);
+                    avc);
     }
   
   rv = 0;
  cleanup:
   __list_iterator_destroy(attrvals_itr);
+  __list_destroy(tmpattrlist);
   return rv;
 }
 
@@ -349,8 +386,14 @@ strsep (char **stringp, const char *delim)
  */
 static int 
 _parse_line(genders_t handle, 
+	    int *numattrs,
+	    int *maxattrs,
+	    int *maxnodelen,
+	    int *maxattrlen,
+	    int *maxvallen,
 	    List nodeslist, 
 	    List attrvalslist, 
+	    List attrslist,
             hash_t *node_index,
             int *node_index_size,
             hash_t *attr_index,
@@ -362,7 +405,7 @@ _parse_line(genders_t handle,
 {
   char *temp, *nodenames, *node = NULL;
   int max_n_subst_vallen = 0, line_maxnodelen = 0, rv = -1;
-  List attrvals = NULL;
+  genders_attrvals_container_t avc = NULL;
   hostlist_t hl = NULL;
   hostlist_iterator_t hlitr = NULL;
 
@@ -398,7 +441,7 @@ _parse_line(genders_t handle,
   /* if strsep() sets line == NULL, line has no attributes */
   if (line) 
     {
-      char *attr, *linebuf;
+      char *attr;
 
       /* move forward to attributes */
       while(isspace(*line))  
@@ -420,10 +463,14 @@ _parse_line(genders_t handle,
 	      goto cleanup;
 	    }
 
-	  __list_create(attrvals, _genders_list_free_genders_attrval);
+	  __xmalloc(avc,
+		    genders_attrvals_container_t,
+		    sizeof(struct genders_attrvals_container));
+	  __list_create(avc->attrvals, _genders_list_free_genders_attrval);
+	  avc->index = list_count(handle->attrvalslist);
 	  
 	  /* parse attributes */
-	  attr = strtok_r(line,",\0",&linebuf);
+	  attr = strsep(&line, ",");
 	  while (attr) 
 	    {
 	      char *val = NULL;
@@ -446,16 +493,40 @@ _parse_line(genders_t handle,
                 }
 #endif
 	
+	      if (!strlen(attr))
+		{
+		  if (line_num > 0) 
+		    {
+		      fprintf(stream, "Line %d: empty string attribute listed\n", line_num);
+		      rv = 1;
+		    }
+		  handle->errnum = GENDERS_ERR_PARSE;
+		  goto cleanup;
+		}
+
+	      if (val && !strlen(val))
+		{
+		  if (line_num > 0) 
+		    {
+		      fprintf(stream, "Line %d: no value specified for attribute \"%s\"\n",
+			      line_num, attr);
+		      rv = 1;
+		    }
+		  handle->errnum = GENDERS_ERR_PARSE;
+		  goto cleanup;
+		}
+
               /* achu: No need to check if there are duplicate
                * attributes within this line of the file.  Will be
                * caught during duplicate attribute checks within each
                * node below.
                */
       
-	      if (_insert_attrval(handle, attrvals, attr, val) < 0)
+	      if (_insert_attrval(handle, avc, attr, val) < 0)
 		goto cleanup;
 	      
               if ((insert_count = _insert_attr(handle,
+					       attrslist,
                                                attr_index,
                                                attr_index_size,
                                                attr)) < 0)
@@ -463,20 +534,19 @@ _parse_line(genders_t handle,
 		  
 	      if (!line_num) 
 		{
-		  handle->numattrs += insert_count;
-		  handle->maxattrlen = GENDERS_MAX(strlen(attr), handle->maxattrlen);
+		  (*numattrs) += insert_count;
+		  (*maxattrlen) = GENDERS_MAX(strlen(attr), (*maxattrlen));
 
 		  if (val) 
 		    {
 		      if (strstr(val, "%n") && !strstr(val, "%%n"))
 			max_n_subst_vallen = strlen(val);
 		      else
-			handle->maxvallen = GENDERS_MAX(strlen(val), 
-							handle->maxvallen);
+			(*maxvallen) = GENDERS_MAX(strlen(val), (*maxvallen));
 		    }
 		}
 	      
-	      attr = strtok_r(NULL,",\0",&linebuf);
+	      attr = strsep(&line, ",");
 	    }
 	}
     }
@@ -517,24 +587,24 @@ _parse_line(genders_t handle,
                              node)))
 	goto cleanup;
       
-      if (attrvals) 
+      if (avc) 
 	{
 	  if ((rv = _attr_node_processsing(handle,
                                            n,
-                                           attrvals,
+                                           avc,
                                            attr_index,
                                            line_num,
                                            stream)) != 0)
 	    goto cleanup;
-	  
-          __list_append(n->attrlist, attrvals);
-	  n->attrcount += list_count(attrvals);
+
+          __list_append(n->attrlist, avc);
+	  n->attrcount += list_count(avc->attrvals);
 	}
       
       if (!line_num) 
 	{
-	  handle->maxattrs = GENDERS_MAX(n->attrcount, handle->maxattrs);
-	  handle->maxnodelen = GENDERS_MAX(strlen(node), handle->maxnodelen);
+	  (*maxattrs) = GENDERS_MAX(n->attrcount, (*maxattrs));
+	  (*maxnodelen) = GENDERS_MAX(strlen(node), (*maxnodelen));
 	  line_maxnodelen = GENDERS_MAX(strlen(node), line_maxnodelen);
 	}
       
@@ -544,21 +614,25 @@ _parse_line(genders_t handle,
   
   /* %n substitution found on this line, update maxvallen */
   if (!line_num && max_n_subst_vallen)
-    handle->maxvallen = GENDERS_MAX(max_n_subst_vallen - 2 + line_maxnodelen,
-				    handle->maxvallen);
+    (*maxvallen) = GENDERS_MAX(max_n_subst_vallen - 2 + line_maxnodelen,
+			       (*maxvallen));
   
   /* Append at the very end, so cleanup area cleaner */
-  if (attrvals) 
+  if (avc) 
     {
-      __list_append(attrvalslist, attrvals);
-      attrvals = NULL;
+      __list_append(attrvalslist, avc);
+      avc = NULL;
     }
   
   rv = 0;
  cleanup:
   __hostlist_iterator_destroy(hlitr);
   __hostlist_destroy(hl);
-  __list_destroy(attrvals);
+  if (avc)
+    {
+      __list_destroy(avc->attrvals);
+      free(avc);
+    }
   free(node);
   return rv;
 }
@@ -566,8 +640,14 @@ _parse_line(genders_t handle,
 int
 _genders_open_and_parse(genders_t handle,
 			const char *filename,
+			int *numattrs,
+			int *maxattrs,
+			int *maxnodelen,
+			int *maxattrlen,
+			int *maxvallen,
 			List nodeslist,
 			List attrvalslist,
+			List attrslist,
                         hash_t *node_index,
                         int *node_index_size,
                         hash_t *attr_index,
@@ -582,7 +662,7 @@ _genders_open_and_parse(genders_t handle,
   int len, errcount = 0, fd = -1, rv = -1, line_count = 1, parsed_nodes = 0;
   char buf[GENDERS_BUFLEN];
 
-  if (!filename)
+  if (!filename || !strlen(filename))
     filename = GENDERS_DEFAULT_FILE;
 
   if ((fd = open(filename, O_RDONLY)) < 0) 
@@ -597,8 +677,14 @@ _genders_open_and_parse(genders_t handle,
       int bug_count;
 
       if ((bug_count = _parse_line(handle, 
+				   numattrs,
+				   maxattrs,
+				   maxnodelen,
+				   maxattrlen,
+				   maxvallen,				   
 				   nodeslist, 
 				   attrvalslist, 
+				   attrslist,
                                    node_index,
                                    node_index_size,
                                    attr_index,
