@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: genders_util.h,v 1.7 2010/02/02 00:04:34 chu11 Exp $
+ *  $Id: genders_util.h,v 1.7 2010-02-02 00:04:34 chu11 Exp $
  *****************************************************************************
  *  Copyright (C) 2007-2010 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2001-2007 The Regents of the University of California.
@@ -259,4 +259,15 @@ int _genders_rehash(genders_t handle,
                     hash_t *hash_ptr,
                     int *hash_size);
 
+/*
+ * _genders_hash_copy
+ *
+ * Copy contents from src hash to dest hash.
+ *
+ * Returns 0 on success, -1 on error.
+ */
+int _genders_hash_copy(genders_t handle,
+		       hash_t *hash_src,
+		       hash_t *hash_dest);
+
 #endif /* _GENDERS_COMMON_H */
