@@ -40,9 +40,12 @@
 #if HAVE_GETOPT_H 
 #include <getopt.h>
 #endif /* HAVE_GETOPT_H */
+#include <errno.h>
 
 #include "genders.h"
+#include "hash.h"
 #include "hostlist.h"
+#include "list.h"
 
 #if HAVE_GETOPT_LONG
 #define GETOPT(ac,av,opt,lopt) getopt_long(ac,av,opt,lopt,NULL)
@@ -50,7 +53,10 @@
 #define GETOPT(ac,av,opt,lopt) getopt(ac,av,opt)
 #endif
 
-#define OPTIONS "cnsqX:vQVUlf:kd:"
+#define OPTIONS "cnsqX:AvQVUlf:kd:e"
+
+/* an impossible attribute */
+#define NOATTRSFLAG "=,,=,,=,,=,,="
 
 #if HAVE_GETOPT_LONG
 static struct option longopts[] = {
@@ -59,6 +65,7 @@ static struct option longopts[] = {
     { "queryspace", 0, 0, 'n' },
     { "query", 0, 0, 'q' },
     { "excludequery", 1, 0, 'X'},
+    { "allnodes", 0, 0, 'A' },
     { "value", 0, 0, 'v' },
     { "testquery", 0, 0, 'Q' },
     { "values", 0, 0, 'V' },
@@ -67,6 +74,8 @@ static struct option longopts[] = {
     { "filename", 1, 0, 'f' },
     { "parse-check", 0, 0, 'k'},
     { "diff", 1, 0, 'd'},
+    { "expand", 0, 0, 'e'},
+    { "compress", 0, 0, 'C'},
     { 0,0,0,0 },
 };
 #endif
@@ -80,6 +89,8 @@ static void list_nodes(genders_t gp, char *attr, char *excludequery, fmt_t fmt);
 static void list_attrs(genders_t gp, char *node);
 static void usage(void);
 static void diff_genders(char *db1, char *db2);
+static void expand(genders_t gp);
+static void compress(genders_t gp);
 
 /* Utility functions */
 static int _gend_error_exit(genders_t gp, char *msg);
@@ -92,12 +103,14 @@ static char *_node_create(genders_t gp);
 static char *_attr_create(genders_t gp);
 #endif
 
+#define HOSTLIST_BUFLEN 1024
+
 int
 main(int argc, char *argv[])
 {
     int c, errors;
-    int lopt = 0, qopt = 0, Xopt = 0, vopt = 0, Qopt = 0,
-      Vopt = 0, Uopt = 0, kopt = 0, dopt = 0;
+    int Aopt = 0, lopt = 0, qopt = 0, Xopt = 0, vopt = 0, Qopt = 0,
+      Vopt = 0, Uopt = 0, kopt = 0, dopt = 0, eopt = 0, Copt = 0;
     char *filename = GENDERS_DEFAULT_FILE;
     char *dfilename = NULL;
     char *excludequery = NULL;
@@ -126,6 +139,9 @@ main(int argc, char *argv[])
             excludequery = optarg;
             Xopt = 1;
             break;
+	case 'A':   /* --allnodes */
+	    Aopt = 1;
+	    break;
         case 'v':   /* --value */
             vopt = 1;
             break;
@@ -151,6 +167,12 @@ main(int argc, char *argv[])
             dopt = 1;
             dfilename = optarg;
             break;
+	case 'e':   /* --expand */
+	    eopt = 1;
+	    break;
+	case 'C':   /* --compress */
+	    Copt = 1;
+	    break;
         default:
             usage();
             break;
@@ -160,33 +182,52 @@ main(int argc, char *argv[])
     /* check parameter inputs */
 
     /* specify correct option combinations */
-    if ((qopt + Qopt + Vopt + lopt + kopt + dopt) > 1)
+    if ((qopt + Qopt + Vopt + lopt + kopt + dopt + eopt + Copt) > 1)
         usage();
 
-    if ((qopt || Qopt || Vopt || lopt || kopt || dopt) && vopt)
+    if ((qopt
+	 || Qopt
+	 || Vopt
+	 || lopt
+	 || kopt
+	 || dopt
+	 || eopt
+	 || Copt)
+	&& vopt)
         usage();
 
+    if (Aopt && !qopt) {
+        qfmt = FMT_HOSTLIST;
+	qopt = 1;
+    }
+
     if (!qopt && Xopt)
-      usage();
+        usage();
 
     if (!Vopt && Uopt)
-      usage();
+        usage();
 
     /* specified correctly number of arguments */
-    if ((qopt && optind != (argc - 1))
+    if ((qopt 
+	 && ((!Aopt && optind != (argc - 1))
+	     || (Aopt && optind != argc))) 
         || (!qopt
             && !Qopt
             && !Vopt
             && !lopt
             && !kopt
             && !dopt
+	    && !eopt
+	    && !Copt
             && (optind != (argc - 1) && optind != (argc - 2)))
         || (Qopt && (optind != (argc - 1) && optind != (argc - 2)))
         || (Vopt && optind != (argc - 1))
         || (lopt && (optind != argc && optind != (argc - 1)))
         || (kopt && optind != argc)
-        || (dopt && optind != argc))
-      usage();
+        || (dopt && optind != argc)
+        || (eopt && optind != argc)
+        || (Copt && optind != argc))
+        usage();
 
     /* genders database diff */
     if (dopt) {
@@ -214,12 +255,28 @@ main(int argc, char *argv[])
     if (genders_load_data(gp, filename) < 0)
         _gend_error_exit(gp, filename);
 
-    /* Usage 1: list nodes with specified attribute. */
+    /* expand */
+    if (eopt) {
+        expand(gp);
+	exit(0);
+    }
+
+    /* compress */
+    if (Copt) {
+        compress(gp);
+	exit(0);
+    }
+
+    /* Usage 1: list nodes with specified attribute, or all nodes */
     if (qopt) {
         char *query;
 
-        query = argv[optind++];
-        list_nodes(gp, query, excludequery, qfmt);
+	if (Aopt)
+	    list_nodes(gp, NULL, NULL, qfmt); 
+	else {
+	    query = argv[optind++];
+	    list_nodes(gp, query, excludequery, qfmt);
+	}
 
         exit(0);
     }
@@ -470,12 +527,15 @@ usage(void)
 {
     fprintf(stderr,
         "Usage: nodeattr [-f genders] [-q|-c|-n|-s] [-X exclude_query] query\n"
+        "or     nodeattr [-f genders] [-q|-c|-n|-s] -A\n"
         "or     nodeattr [-f genders] [-v] [node] attr[=val]\n"
         "or     nodeattr [-f genders] -Q [node] query\n"
         "or     nodeattr [-f genders] -V [-U] attr\n"   
         "or     nodeattr [-f genders] -l [node]\n"
         "or     nodeattr [-f genders] -k\n"
-        "or     nodeattr [-f genders] -d genders\n"    
+        "or     nodeattr [-f genders] -d genders\n"
+	"or     nodeattr [-f genders] --expand\n"
+	"or     nodeattr [-f genders] --compress\n"
             );
     exit(1);
 }
@@ -743,6 +803,427 @@ diff_genders(char *filename, char *dfilename)
         return;
 }
 
+static void
+expand(genders_t gp)
+{
+    char **nodes, **attrs, **vals;
+    int nodeslen, attrslen, valslen;
+    int nodescount, attrscount;
+    unsigned int maxnodenamelen = 0;
+    hostlist_t hl = NULL;
+    hostlist_iterator_t hlitr = NULL;
+    char *node;
+    int i, j;
+
+    if ((nodeslen = genders_nodelist_create(gp, &nodes)) < 0)
+        _gend_error_exit(gp, "genders_nodelist_create");
+
+    if ((attrslen = genders_attrlist_create(gp, &attrs)) < 0)
+        _gend_error_exit(gp, "genders_attrlist_create");
+
+    if ((valslen = genders_vallist_create(gp, &vals)) < 0)
+        _gend_error_exit(gp, "genders_vallist_create");
+
+    if ((nodescount = genders_getnodes(gp, nodes, nodeslen, NULL, NULL)) < 0)
+        _gend_error_exit(gp, "genders_getnodes");
+
+    /* We use the hostlist as a cheap mechanism to sort the node names
+     * before outputting them
+     */
+
+    if (!(hl = hostlist_create(NULL))) {
+        fprintf(stderr, "hostlist_create: %s\n", strerror(errno));
+	exit(1);
+    }
+
+    for (i = 0; i < nodescount; i++) {
+        unsigned int tmp = strlen(nodes[i]);
+	if (tmp > maxnodenamelen) {
+	    maxnodenamelen = tmp;
+	}
+
+	if (!hostlist_push(hl, nodes[i])) {
+	    fprintf(stderr, "hostlist_push: %s\n", strerror(errno));
+	    exit(1);
+	}
+    }
+
+    hostlist_sort(hl);
+
+    if (!(hlitr = hostlist_iterator_create(hl))) {
+	fprintf(stderr, "hostlist_iterator_create: %s\n", strerror(errno));
+	exit(1);
+    }
+
+    while ((node = hostlist_next(hlitr))) {
+        if (genders_attrlist_clear(gp, attrs) < 0)
+	    _gend_error_exit(gp, "genders_attrlist_clear");
+
+        if (genders_vallist_clear(gp, vals) < 0)
+	    _gend_error_exit(gp, "genders_vallist_clear");
+
+	if ((attrscount = genders_getattr(gp, attrs, vals, attrslen, node)) < 0)
+	    _gend_error_exit(gp, "genders_getattr");
+	
+	printf("%s", node);
+	if (attrscount) {
+	    unsigned int numspace = maxnodenamelen - strlen(node);
+	    for (j = 0; j < numspace; j++)
+		printf(" ");
+	    printf(" ");
+	}
+
+	for (j = 0 ; j < attrscount; j++) {
+	    if (j)
+	        printf(",");
+
+	    if (strlen(vals[j]))
+	        printf("%s=%s", attrs[j], vals[j]); 
+	    else
+	        printf("%s", attrs[j]);
+	}
+
+	printf("\n");
+	free(node);
+    }
+
+    genders_nodelist_destroy(gp, nodes);
+    genders_attrlist_destroy(gp, attrs);
+    genders_vallist_destroy(gp, vals);
+    hostlist_destroy(hl);
+}
+
+struct hosts_data {
+    char *key;
+    hostlist_t hl;
+};
+
+struct attr_list {
+    char *hostrange;
+    List l;
+};
+
+struct store_hostrange_data {
+    List hlist;
+    unsigned int maxhostrangelen;
+};
+
+static void
+_hosts_data_del(void *data)
+{
+    struct hosts_data *hd = (struct hosts_data *)data;
+
+    free(hd->key);
+    hostlist_destroy(hd->hl);
+    free(hd);
+}
+
+static void
+_attr_list_del(void *data)
+{
+    struct attr_list *al = (struct attr_list *)data;
+
+    free(al->hostrange);
+    list_destroy(al->l);
+    free(al);
+}
+
+static void
+_hash_attrval(hash_t hattr, char *node, char *attr, char *val)
+{
+    struct hosts_data *hd = NULL;
+    char *hashkey = NULL;
+    int keylen, attrlen, vallen;
+    
+    assert(hattr && node && attr && val);
+
+    attrlen = strlen(attr);
+    vallen = strlen(val);
+    keylen = attrlen + vallen;
+
+    /* for equal sign */
+    if (vallen)
+	keylen++;
+
+    /* for NUL char */
+    keylen++;
+    
+    if (!(hashkey = (char *)malloc(keylen))) {
+	fprintf(stderr, "malloc: %s\n", strerror(errno));
+	exit(1);
+    }
+
+    if (vallen)
+	snprintf(hashkey, keylen, "%s=%s", attr, val);
+    else
+	snprintf(hashkey, keylen, "%s", attr);
+
+    if (!(hd = hash_find(hattr, hashkey))) {
+	if (!(hd = (struct hosts_data *)malloc(sizeof(struct hosts_data)))) {
+	    fprintf(stderr, "malloc: %s\n", strerror(errno));
+	    exit(1);
+	}
+
+	hd->key = hashkey;
+	if (!(hd->hl = hostlist_create(NULL))) {
+	    fprintf(stderr, "hostlist_create: %s\n", strerror(errno));
+	    exit(1);
+	}
+
+	if (!hash_insert(hattr, hd->key, hd)) {
+	    fprintf(stderr, "hash_insert: %s\n", strerror(errno));
+	    exit(1);
+	}
+    }
+    else
+	free(hashkey);
+
+    if (!hostlist_push(hd->hl, node)) {
+	fprintf(stderr, "hostlist_push: %s\n", strerror(errno));
+	exit(1);
+    }
+}
+
+static int
+_hash_hostrange(void *data, const void *key, void *arg)
+{
+    struct hosts_data *hd = (struct hosts_data *)data;
+    hash_t *hrange = (hash_t *)arg;
+    char hostrange[HOSTLIST_BUFLEN + 1];
+    struct attr_list *al;
+
+    memset(hostrange, '\0', HOSTLIST_BUFLEN + 1);
+
+    hostlist_sort(hd->hl);
+
+    if (hostlist_ranged_string(hd->hl, HOSTLIST_BUFLEN, hostrange) < 0) {
+        fprintf(stderr, "hostlist_ranged_string: %s\n", strerror(errno));
+	exit(1);
+    }
+
+    if (!(al = hash_find(*hrange, hostrange))) {
+        if (!(al = (struct attr_list *)malloc(sizeof(struct attr_list)))) {
+	    fprintf(stderr, "malloc: %s\n", strerror(errno));
+	    exit(1);
+	}
+	
+	if (!(al->hostrange = strdup(hostrange))) {
+	    fprintf(stderr, "strdup: %s\n", strerror(errno));
+	    exit(1);
+	}
+	
+	if (!(al->l = list_create(NULL))) {
+	    fprintf(stderr, "list_create: %s\n", strerror(errno));
+	    exit(1);
+	}
+
+	if (!hash_insert(*hrange, al->hostrange, al)) {
+	    fprintf(stderr, "hash_insert: %s\n", strerror(errno));
+	    exit(1);
+	}
+    }
+
+    if (!list_append(al->l, hd->key)) {
+        fprintf(stderr, "list_append: %s\n", strerror(errno));
+	exit(1);
+    }
+
+    return 0;
+}
+
+static int
+_store_hostrange(void *data, const void *key, void *arg)
+{
+    struct attr_list *al = (struct attr_list *)data;
+    struct store_hostrange_data *shd = (struct store_hostrange_data *)arg;
+    unsigned int len;
+
+    if (!list_append(shd->hlist, al)) {
+        fprintf(stderr, "list_append: %s\n", strerror(errno));
+	exit(1);
+    }
+
+    len = strlen(al->hostrange);
+    if (len > shd->maxhostrangelen)
+	shd->maxhostrangelen = len;
+
+    return 0;
+}
+
+static int
+_hostrange_cmp(void *x, void *y)
+{
+    struct attr_list *al1 = (struct attr_list *)x;
+    struct attr_list *al2 = (struct attr_list *)y;
+
+    if (strlen(al1->hostrange) < strlen(al2->hostrange))
+	return 1;
+    else if (strlen(al1->hostrange) > strlen(al2->hostrange))
+	return -1;
+    else
+	return 0;
+}
+
+static int
+_output_hostrange(void *x, void *arg)
+{
+    struct attr_list *al = (struct attr_list *)x;
+    unsigned int maxhostrangelen = *(unsigned int *)arg;
+    char *attrval;
+    ListIterator litr;
+    int lcount, count = 0;
+    unsigned int numspace;
+    int i;
+
+    printf("%s", al->hostrange);
+    numspace = maxhostrangelen - strlen(al->hostrange);
+    for (i = 0; i < numspace; i++)
+	printf(" ");
+    printf(" ");
+    
+    lcount = list_count(al->l);
+
+    if (!(litr = list_iterator_create(al->l))) {
+        fprintf(stderr, "list_iterator_create: %s\n", strerror(errno));
+	exit(1);
+    }
+    
+    while ((attrval = list_next(litr))) {
+
+	if (!strcmp(attrval, NOATTRSFLAG))
+	    continue;
+
+        printf("%s", attrval);
+	count++;
+	if (lcount != count)
+	    printf(",");
+    }
+
+    printf("\n");
+
+    list_iterator_destroy(litr);
+    return 0;
+}
+
+static void
+compress(genders_t gp)
+{
+    char **nodes, **attrs, **vals;
+    int nodeslen, attrslen, valslen;
+    int nodescount, attrscount;
+    int numnodes, numattrs;
+    hash_t hattr = NULL;
+    hash_t hrange = NULL;
+    List hlist = NULL;
+    struct store_hostrange_data shd;
+    int i, j;
+
+    /* The basic idea behind this algorithm is that we will find every
+     * host that contains an attr or attr=val combination.
+     *
+     * Then, we will find every attr or attr=val combination with the
+     * same sets of hosts, than output a compressed hostrange output
+     * for those hosts with every appropriate attr/attr=val.
+     */
+
+    /* need to treat values w/ raw inputs in order to compress */
+    if (genders_set_flags(gp, GENDERS_FLAG_RAW_VALUES) < 0)
+        _gend_error_exit(gp, "genders_set_flags");
+
+    if ((numnodes = genders_getnumnodes(gp)) < 0)
+        _gend_error_exit(gp, "genders_getnumnodes");
+
+    if ((numattrs = genders_getnumattrs(gp)) < 0)
+        _gend_error_exit(gp, "genders_getnumattrs");
+
+    /* numattrs + 1, in case numattrs == 0
+     *
+     * (numattrs + 1) * 4, is an estimate on attribute=value pair
+     * types, b/c we are keying off attr=val pairs, not just the
+     * attribute name.
+     */
+    if (!(hattr = hash_create((numattrs + 1)*4, 
+			      (hash_key_f)hash_key_string,
+			      (hash_cmp_f)strcmp,
+			      _hosts_data_del))) {
+        fprintf(stderr, "hash_create: %s\n", strerror(errno));
+	exit(1);
+    }
+
+    if ((nodeslen = genders_nodelist_create(gp, &nodes)) < 0)
+        _gend_error_exit(gp, "genders_nodelist_create");
+
+    if ((attrslen = genders_attrlist_create(gp, &attrs)) < 0)
+        _gend_error_exit(gp, "genders_attrlist_create");
+
+    if ((valslen = genders_vallist_create(gp, &vals)) < 0)
+        _gend_error_exit(gp, "genders_vallist_create");
+
+    if ((nodescount = genders_getnodes(gp, nodes, nodeslen, NULL, NULL)) < 0)
+        _gend_error_exit(gp, "genders_getnodes");
+
+    for (i = 0; i < nodescount; i++) {
+        if (genders_attrlist_clear(gp, attrs) < 0)
+	    _gend_error_exit(gp, "genders_attrlist_clear");
+
+        if (genders_vallist_clear(gp, vals) < 0)
+	    _gend_error_exit(gp, "genders_vallist_clear");
+
+	if ((attrscount = genders_getattr(gp, attrs, vals, attrslen, nodes[i])) < 0)
+	    _gend_error_exit(gp, "genders_getattr");
+
+	if (!attrscount) {
+	    _hash_attrval(hattr, nodes[i], NOATTRSFLAG, "");
+	    continue;
+	}
+	
+	for (j = 0 ; j < attrscount; j++)
+	    _hash_attrval(hattr, nodes[i], attrs[j], vals[j]);
+    }
+
+    /* Now, find all the common attributes for a particular hostrange */
+
+    if (!(hrange = hash_create(numnodes, 
+			      (hash_key_f)hash_key_string,
+			      (hash_cmp_f)strcmp,
+			      _attr_list_del))) {
+        fprintf(stderr, "hash_create: %s\n", strerror(errno));
+	exit(1);
+    }
+
+    if (hash_for_each(hattr, _hash_hostrange, &hrange) < 0) {
+        fprintf(stderr, "hash_for_each: %s\n", strerror(errno));
+	exit(1);
+    }
+
+    if (!(hlist = list_create(NULL))) {
+        fprintf(stderr, "list_create: %s\n", strerror(errno));
+	exit(1);
+    }
+
+    shd.hlist = hlist;
+    shd.maxhostrangelen = 0;
+
+    if (hash_for_each(hrange, _store_hostrange, &shd) < 0) {
+        fprintf(stderr, "hash_for_each: %s\n", strerror(errno));
+	exit(1);
+    }
+
+    list_sort(hlist, _hostrange_cmp);
+
+    if (list_for_each(hlist, _output_hostrange, &shd.maxhostrangelen) < 0) {
+	fprintf(stderr, "list_for_each: %s\n", strerror(errno));
+	exit(1);
+    }
+
+    genders_nodelist_destroy(gp, nodes);
+    genders_attrlist_destroy(gp, attrs);
+    genders_vallist_destroy(gp, vals);
+    hash_destroy(hattr);
+    hash_destroy(hrange);
+    list_destroy(hlist);
+}
+
 /**
  ** Utility functions
  **/
